from .utils import POSTGRES_USER, POSTGRES_PASSWORD, POSTGRES_DB


if POSTGRES_USER == "" or POSTGRES_PASSWORD == "" or POSTGRES_DB == "":
    raise RuntimeError("Postgres env vars not set")


from .utils import async_session, engine, DATABASE_URL

__all__ = ["async_session", "engine", "DATABASE_URL"]



import uuid
import datetime
from typing import Optional
from shared_models.database.utils import Base
from shared_models.enums import SubscriptionType
from sqlalchemy import func, String, ForeignKey, Text
from sqlalchemy.orm import Mapped, mapped_column, relationship
from sqlalchemy.ext.associationproxy import association_proxy, AssociationProxy
from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from .user import User
    from .vacancy import Vacancy


class CompanyUser(Base):
    __tablename__ = "company_user"

    company_id: Mapped[uuid.UUID] = mapped_column(
        ForeignKey("company.id"), primary_key=True
    )
    user_id: Mapped[uuid.UUID] = mapped_column(ForeignKey("user.id"), primary_key=True)
    added_at: Mapped[datetime.datetime] = mapped_column(server_default=func.now())

    company: Mapped["Company"] = relationship("Company", back_populates="company_users")
    user: Mapped["User"] = relationship("User", back_populates="company_user")

    def __init__(self, user: "User"):
        """
        Initialize a CompanyUser object.

        Args:
            user (User): The associated user.
        """
        self.user = user


class Company(Base):
    __tablename__ = "company"

    id: Mapped[uuid.UUID] = mapped_column(primary_key=True, default=uuid.uuid4)
    name: Mapped[str] = mapped_column(String(100))
    created_at: Mapped[datetime.datetime] = mapped_column(server_default=func.now())

    reject_message: Mapped[Optional["CompanyRejectMessage"]] = relationship(
        back_populates="company",
        uselist=False,
        cascade="all, delete-orphan",
    )
    accept_message: Mapped[Optional["CompanyAcceptMessage"]] = relationship(
        back_populates="company",
        uselist=False,
        cascade="all, delete-orphan",
    )
    subscription: Mapped[Optional["CompanySubscription"]] = relationship(
        back_populates="company",
        uselist=False,
        cascade="all, delete-orphan",
    )
    company_users: Mapped[list["CompanyUser"]] = relationship(
        back_populates="company", cascade="all, delete-orphan"
    )
    users: AssociationProxy[list["User"]] = association_proxy("company_users", "user")
    vacancies: Mapped[list["Vacancy"]] = relationship(
        back_populates="company", cascade="all, delete-orphan"
    )

    def __init__(self, name: str):
        """
        Initialize a Company object.

        Args:
            name (str): The name of the company.
        """
        self.name = name


class CompanyRejectMessage(Base):
    __tablename__ = "company_reject_message"

    company_id: Mapped[uuid.UUID] = mapped_column(
        ForeignKey("company.id"), primary_key=True
    )
    message: Mapped[str] = mapped_column(Text)
    created_at: Mapped[datetime.datetime] = mapped_column(server_default=func.now())
    updated_at: Mapped[datetime.datetime] = mapped_column(
        server_default=func.now(), onupdate=func.now()
    )

    company: Mapped["Company"] = relationship(back_populates="reject_message")

    def __init__(self, message: str, company: Company):
        """
        Initialize a CompanyRejectMessage object.

        Args:
            message (str): The rejection message.
            company (Company): The associated company.
        """
        self.message = message
        self.company = company


class CompanyAcceptMessage(Base):
    __tablename__ = "company_accept_message"

    company_id: Mapped[uuid.UUID] = mapped_column(
        ForeignKey("company.id"), primary_key=True
    )
    message: Mapped[str] = mapped_column(Text)
    created_at: Mapped[datetime.datetime] = mapped_column(server_default=func.now())
    updated_at: Mapped[datetime.datetime] = mapped_column(
        server_default=func.now(), onupdate=func.now()
    )

    company: Mapped["Company"] = relationship(back_populates="accept_message")

    def __init__(self, message: str, company: Company):
        """
        Initialize a CompanyAcceptMessage object.

        Args:
            message (str): The acceptance message.
            company (Company): The associated company.
        """
        self.message = message
        self.company = company


class CompanySubscription(Base):
    __tablename__ = "company_subscription"

    id: Mapped[uuid.UUID] = mapped_column(primary_key=True, default=uuid.uuid4)
    company_id: Mapped[uuid.UUID] = mapped_column(ForeignKey("company.id"))
    expire_at: Mapped[datetime.datetime] = mapped_column()
    type: Mapped[SubscriptionType] = mapped_column()
    buy_at: Mapped[datetime.datetime] = mapped_column(server_default=func.now())

    company: Mapped["Company"] = relationship(back_populates="subscription")

    def __init__(
        self,
        company: Company,
        expire_at: datetime.datetime,
        type: SubscriptionType,
        buy_at: datetime.datetime | None = None,
    ):
        """
        Initialize a CompanySubscription object.

        Args:
            company (Company): The associated company.
            expire_at (datetime.datetime): The expiration date of the subscription.
            type (SubscriptionType): The type of the subscription.
            buy_at (datetime.datetime | None): The purchase date of the subscription. Defaults to None, which sets it to the current time.
        """
        self.company = company
        self.expire_at = expire_at
        self.type = type
        if buy_at is not None:
            self.buy_at = buy_at



import uuid
import datetime
from shared_models.database.utils import Base
from shared_models.enums import QuestionType, ContentType
from sqlalchemy import func, ForeignKey, Text
from sqlalchemy.orm import Mapped, mapped_column, relationship, validates
from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from .vacancy import Vacancy, VacancyQuestion, TestQuestionOption
    from .user import User


class VacancyResponse(Base):
    __tablename__ = "vacancy_response"

    id: Mapped[uuid.UUID] = mapped_column(primary_key=True, default=uuid.uuid4)
    vacancy_id: Mapped[uuid.UUID] = mapped_column(ForeignKey("vacancy.id"))
    user_id: Mapped[uuid.UUID] = mapped_column(ForeignKey("user.id"))
    created_at: Mapped[datetime.datetime] = mapped_column(server_default=func.now())

    vacancy: Mapped["Vacancy"] = relationship(back_populates="responses")
    user: Mapped["User"] = relationship(back_populates="responses")
    answers: Mapped[list["VacancyQuestionAnswer"]] = relationship(
        back_populates="response", cascade="all, delete-orphan"
    )

    def __init__(
        self,
        vacancy: "Vacancy",
        user: "User",
    ):
        """
        Initializes a VacancyResponse object.

        Args:
            vacancy (Vacancy): The associated vacancy.
            user (User): The associated user.
        """
        self.vacancy = vacancy
        self.user = user


class VacancyQuestionAnswer(Base):
    __tablename__ = "vacancy_question_answer"
    __mapper_args__ = {"polymorphic_on": "type", "polymorphic_abstract": True}

    id: Mapped[uuid.UUID] = mapped_column(primary_key=True, default=uuid.uuid4)
    response_id: Mapped[uuid.UUID] = mapped_column(ForeignKey("vacancy_response.id"))
    question_id: Mapped[uuid.UUID] = mapped_column(ForeignKey("vacancy_question.id"))
    type: Mapped[QuestionType] = mapped_column()

    response: Mapped["VacancyResponse"] = relationship(back_populates="answers")
    question: Mapped["VacancyQuestion"] = relationship(back_populates="answers")

    def __init__(
        self,
        question: "VacancyQuestion",
    ):
        """
        Initializes the object with a given VacancyQuestion.

        Args:
            question (VacancyQuestion): The question associated with this response.
        """
        self.question = question


class OpenQuestionAnswer(VacancyQuestionAnswer):
    __tablename__ = "open_question_answer_info"
    __mapper_args__ = {
        "polymorphic_identity": QuestionType.OPEN,
    }

    id: Mapped[uuid.UUID] = mapped_column(
        ForeignKey("vacancy_question_answer.id"), primary_key=True
    )
    content_type: Mapped[ContentType] = mapped_column()
    text: Mapped[str | None] = mapped_column(Text)
    content_file_id: Mapped[uuid.UUID | None] = mapped_column()

    @validates("content_type")
    def validate_content_type(self, _, content_type):
        if content_type == ContentType.ANY:
            raise ValueError("Content type cannot be ANY for OpenQuestionAnswerInfo")
        return content_type

    @validates("content_file_id", "text")
    def validate_content(self, key, value):
        if (
            key == "content_file_id"
            and self.content_type == ContentType.TEXT
            and value is not None
        ):
            raise ValueError("content_file_id must be None if content_type is TEXT")
        if (
            key == "text"
            and self.content_type != ContentType.TEXT
            and value is not None
        ):
            raise ValueError("text must be None if content_type is not TEXT")
        return value

    def __init__(
        self,
        question: "VacancyQuestion",
        content_type: ContentType,
        text: str | None = None,
        content_file_id: uuid.UUID | None = None,
    ):
        """
        Initializes an OpenQuestionAnswer object.

        Args:
            question (VacancyQuestion): The question associated with this answer.
            content_type (ContentType): The type of content of the answer.
            text (str | None): The text content of the answer. Defaults to None.
            content_file_id (uuid.UUID | None): The id of the content file of the answer. Defaults to None.
        """
        super().__init__(question)
        self.content_type = content_type
        self.text = text
        self.content_file_id = content_file_id


class TestQuestionAnswer(VacancyQuestionAnswer):
    __tablename__ = "test_question_answer_info"
    __mapper_args__ = {
        "polymorphic_identity": QuestionType.TEST,
    }

    id: Mapped[uuid.UUID] = mapped_column(
        ForeignKey("vacancy_question_answer.id"), primary_key=True
    )
    option_id: Mapped[uuid.UUID] = mapped_column(ForeignKey("test_question_option.id"))

    option: Mapped["TestQuestionOption"] = relationship()

    def __init__(
        self,
        question: "VacancyQuestion",
        option: "TestQuestionOption",
    ):
        """
        Initializes a TestQuestionAnswer object.

        Args:
            question (VacancyQuestion): The question associated with this answer.
            option (TestQuestionOption): The option associated with this answer.
        """
        super().__init__(question)
        self.option = option



from .company import (
    Company,
    CompanyUser,
    CompanyAcceptMessage,
    CompanyRejectMessage,
    CompanySubscription,
)
from .user import User, TelegramUser
from .vacancy import (
    Vacancy,
    VacancyQuestion,
    OpenQuestion,
    TestQuestion,
    TestQuestionOption,
)
from .response import (
    VacancyResponse,
    VacancyQuestionAnswer,
    OpenQuestionAnswer,
    TestQuestionAnswer,
)


__all__ = [
    "Company",
    "CompanyUser",
    "CompanyAcceptMessage",
    "CompanyRejectMessage",
    "CompanySubscription",
    "User",
    "TelegramUser",
    "Vacancy",
    "VacancyQuestion",
    "OpenQuestion",
    "TestQuestion",
    "TestQuestionOption",
    "VacancyResponse",
    "VacancyQuestionAnswer",
    "OpenQuestionAnswer",
    "TestQuestionAnswer",
]



import os
from sqlalchemy.orm import DeclarativeBase
from sqlalchemy.ext.asyncio import AsyncAttrs, create_async_engine, async_sessionmaker
from typing import Final
from dotenv import load_dotenv

load_dotenv()

POSTGRES_USER: Final[str] = os.getenv("POSTGRES_USER", "")
POSTGRES_PASSWORD: Final[str] = os.getenv("POSTGRES_PASSWORD", "")
POSTGRES_DB: Final[str] = os.getenv("POSTGRES_DB", "")
POSTGRES_HOST: Final[str] = os.getenv("POSTGRES_HOST", "localhost")
POSTGRES_PORT: Final[int] = int(os.getenv("POSTGRES_PORT", "5432"))

DATABASE_URL: Final[str] = (
    f"postgresql+asyncpg://{POSTGRES_USER}:{POSTGRES_PASSWORD}@{POSTGRES_HOST}:{POSTGRES_PORT}/{POSTGRES_DB}"
)


class Base(AsyncAttrs, DeclarativeBase):
    pass


engine = create_async_engine(DATABASE_URL)
async_session = async_sessionmaker(engine, expire_on_commit=False)



import uuid
import datetime
from shared_models.database.utils import Base
from shared_models.enums import QuestionType, ContentType
from sqlalchemy import func, ForeignKey, Text, String
from sqlalchemy.orm import Mapped, mapped_column, relationship
from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from .company import Company
    from .response import VacancyQuestionAnswer, VacancyResponse


class Vacancy(Base):
    __tablename__ = "vacancy"

    id: Mapped[uuid.UUID] = mapped_column(primary_key=True, default=uuid.uuid4)
    company_id: Mapped[uuid.UUID] = mapped_column(ForeignKey("company.id"))
    position: Mapped[str] = mapped_column(Text)
    experience: Mapped[str] = mapped_column(Text)
    description: Mapped[str] = mapped_column(Text)
    requirements: Mapped[str] = mapped_column(Text)
    created_at: Mapped[datetime.datetime] = mapped_column(server_default=func.now())
    updated_at: Mapped[datetime.datetime] = mapped_column(
        server_default=func.now(), onupdate=func.now()
    )

    company: Mapped["Company"] = relationship(back_populates="vacancies")
    questions: Mapped[list["VacancyQuestion"]] = relationship(
        back_populates="vacancy", cascade="all, delete-orphan"
    )
    responses: Mapped[list["VacancyResponse"]] = relationship(
        back_populates="vacancy", cascade="all, delete-orphan"
    )

    def __init__(
        self,
        company: "Company",
        position: str,
        experience: str,
        description: str,
        requirements: str,
    ):
        """
        Initialize a Vacancy object.

        Args:
            company (Company): The associated company.
            position (str): The position of the vacancy.
            experience (str): The experience required for the vacancy.
            description (str): The description of the vacancy.
            requirements (str): The requirements of the vacancy.
        """
        self.company = company
        self.position = position
        self.experience = experience
        self.description = description
        self.requirements = requirements


class VacancyQuestion(Base):
    __tablename__ = "vacancy_question"
    __mapper_args__ = {"polymorphic_on": "type", "polymorphic_abstract": True}

    id: Mapped[uuid.UUID] = mapped_column(primary_key=True, default=uuid.uuid4)
    vacancy_id: Mapped[uuid.UUID] = mapped_column(ForeignKey("vacancy.id"))
    text: Mapped[str] = mapped_column(Text)
    ordinal_number: Mapped[int] = mapped_column()
    type: Mapped[QuestionType] = mapped_column()

    vacancy: Mapped["Vacancy"] = relationship(back_populates="questions")
    answers: Mapped[list["VacancyQuestionAnswer"]] = relationship(
        back_populates="question", cascade="all, delete-orphan"
    )

    def __init__(self, text: str, ordinal_number: int):
        """
        Initialize an OpenQuestion object.

        Args:
            text (str): The text of the question.
            ordinal_number (int): The ordinal number of the question.
        """
        self.text = text
        self.ordinal_number = ordinal_number


class OpenQuestion(VacancyQuestion):
    __tablename__ = "open_question_info"
    __mapper_args__ = {
        "polymorphic_identity": QuestionType.OPEN,
    }

    id: Mapped[uuid.UUID] = mapped_column(
        ForeignKey("vacancy_question.id"), primary_key=True
    )
    response_format: Mapped[ContentType] = mapped_column()

    def __init__(
        self,
        text: str,
        ordinal_number: int,
        response_format: ContentType,
    ):
        """
        Initializes an OpenQuestion object.

        Args:
            text (str): The text of the question.
            ordinal_number (int): The ordinal number of the question.
            response_format (ContentType): The response format of the question.
        """
        super().__init__(text, ordinal_number)
        self.response_format = response_format


class TestQuestion(VacancyQuestion):
    __tablename__ = "test_question_info"
    __mapper_args__ = {
        "polymorphic_identity": QuestionType.TEST,
    }

    id: Mapped[uuid.UUID] = mapped_column(
        ForeignKey("vacancy_question.id"), primary_key=True
    )
    continue_if_incorrect: Mapped[bool] = mapped_column()

    options: Mapped[list["TestQuestionOption"]] = relationship(
        back_populates="question"
    )

    def __init__(
        self,
        text: str,
        ordinal_number: int,
        continue_if_incorrect: bool,
    ):
        """
        Initializes a TestQuestion object.

        Args:
            text (str): The text of the question.
            ordinal_number (int): The ordinal number of the question.
            continue_if_incorrect (bool): Whether to continue if the answer is incorrect.
        """
        super().__init__(text, ordinal_number)
        self.continue_if_incorrect = continue_if_incorrect


class TestQuestionOption(Base):
    __tablename__ = "test_question_option"

    id: Mapped[uuid.UUID] = mapped_column(primary_key=True, default=uuid.uuid4)
    question_id: Mapped[uuid.UUID] = mapped_column(ForeignKey("test_question_info.id"))
    option: Mapped[str] = mapped_column(String(100))
    is_correct: Mapped[bool] = mapped_column()

    question: Mapped["TestQuestion"] = relationship(back_populates="options")

    def __init__(self, option: str, is_correct: bool):
        """
        Initializes a TestQuestionOption object.

        Args:
            option (str): The option text.
            is_correct (bool): Whether the option is correct.
        """
        self.option = option
        self.is_correct = is_correct



import uuid
import datetime
from typing import Optional
from shared_models.database.utils import Base
from sqlalchemy import func, String, ForeignKey
from sqlalchemy.orm import Mapped, mapped_column, relationship
from sqlalchemy.ext.associationproxy import association_proxy, AssociationProxy
from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from .company import Company, CompanyUser
    from .response import VacancyResponse


class User(Base):
    __tablename__ = "user"

    id: Mapped[uuid.UUID] = mapped_column(primary_key=True, default=uuid.uuid4)
    name: Mapped[str | None] = mapped_column(String(100))
    created_at: Mapped[datetime.datetime] = mapped_column(server_default=func.now())

    telegram_account: Mapped[Optional["TelegramUser"]] = relationship(
        back_populates="user",
        uselist=False,
        cascade="all, delete-orphan",
    )
    company_user: Mapped[Optional["CompanyUser"]] = relationship(
        back_populates="user", uselist=False, cascade="all, delete-orphan"
    )
    company: AssociationProxy[Optional["Company"]] = association_proxy(
        "company_user", "company"
    )
    responses: Mapped[list["VacancyResponse"]] = relationship(
        back_populates="user", cascade="all, delete-orphan"
    )

    def __init__(self, name: str | None = None):
        """
        Initialize a User object.

        Args:
            name (str | None): The name of the user. Defaults to None.
        """
        self.name = name


class TelegramUser(Base):
    __tablename__ = "telegram_user"

    telegram_id: Mapped[int] = mapped_column(primary_key=True)
    user_id: Mapped[uuid.UUID] = mapped_column(ForeignKey("user.id"))
    username: Mapped[str | None] = mapped_column(String(40))

    user: Mapped[User] = relationship(back_populates="telegram_account")

    def __init__(self, telegram_id: int, user: User, username: str | None = None):
        """
        Initialize a TelegramUser object.

        Args:
            telegram_id (int): The telegram id of the user.
            user (User): The user object associated with the telegram user.
            username (str | None): The username of the telegram user. Defaults to None.
        """
        self.telegram_id = telegram_id
        self.username = username
        self.user = user
